﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace Text_Adventure_Tutorial
{
	public partial class MainForm : Form
	{
        #region Class variables
		private Location currentLocation;
		private string NorthExit;
		private string EastExit;
		private string SouthExit;
		private string WestExit;
		private Dictionary<string, Location> dictLocation = new Dictionary<string, Location>();  // Dictionary of all locations in the game
		private Dictionary<string,Item> dictItems = new Dictionary<string,Item>();				// Dictionary of all Items in the game
        #endregion
		public MainForm()
		{
			InitializeComponent();
			this.CenterToScreen();
		}
		#region Event Handlers
		private void MainFormLoad(object sender, EventArgs e)						{CreateGame();}
		private void BtnDropClick(object sender, EventArgs e) 						{DropItem();}
		private void BtnNorthClick(object sender, EventArgs e)						{Go("north");}
		private void BtnEastClick(object sender, EventArgs e)						{Go("east");}
		private void BtnSouthClick(object sender, EventArgs e)						{Go("south");}
		private void BtnWestClick(object sender, EventArgs e)						{Go("west");}
		private void BtnExamineClick(object sender, EventArgs e)					{ExamineItem(lstLocation);}
		private void BtnTakeClick(object sender, EventArgs e)						{TakeItem();}
		private void BtnHoldClick(object sender, EventArgs e)						{HoldItem();}
		private void BtnReturnClick(object sender, EventArgs e)						{ReturnItem();}
		private void BtnUseClick(object sender, EventArgs e)						{UseItem();}
		private void BtnExamineInventoryClick(object sender, EventArgs e)			{ExamineItem(lstInventory);}
		#endregion
		#region Methods
		
		private void CheckExit(Location exitLoc)
		{
			if(exitLoc.ItemRequired == null)
			{
				currentLocation = exitLoc;
			}
			else
			{
				if(exitLoc.ItemRequired == Player.ItemInHand)
				{
					currentLocation = exitLoc;	
				}
				else
				{
					DisplayStory($"\nYou need the {exitLoc.ItemRequired.Name} in your hand to enter {exitLoc.DisplayName}");
				}
			}
		}
		private void CreateGame()
		{
			// create 6 Item objects and a Weapon object
			
			dictItems.Add("torch", new Item("torch", "a flaming wooden torch. It probably should not be in this adventure.."));
			dictItems.Add("LED torch",new Item("LED torch", "a new-fangled battery powered monstrosity"));
			dictItems.Add("iron key",new Item("iron key", "a rusty old key suitable for a dungeon door"));
			dictItems.Add("hotel card", new Item("hotel card", "a plastic key card with magnetic strip. Probably not much use in a dungeon"));
			// add a Weapon to the same Dictionary, as Weapon is a sub-class of Item
			dictItems.Add("knife", new Weapon("knife", "a poor quality kitchen knife from the Pound Shop", 100));
			// add a Potion to the same Dictionary, as Potion is a sub-class of Item
			dictItems.Add("aspirin", new Potion("aspirin", "a tablet useful after a late night...", 100));
			// add a Note to the same Dictionary, as Potion is a sub-class of Item
			dictItems.Add("note", new Note("note", "a hotel provided acrylic plaque", "Welcome to the adventure. Don't forget the hotel key card before you leave"));
			/*
			world plan:
										0  = bedroom
			|---|---|---|---|			1  = coridoor
			| 0   1   2 | 9 |			2  = lift
			|---|- -|---|   |			3  = hallway
			    | 3 |||||   |			4  = diner
			|---|- -|||||     10		5  = store	
			| 5   4 |||||   |			6  = bar
			|---|- -|---|   |			7  = games
			    | 6   7     |			8  = wc
			    |---|- -|---|			9  = reception
			        | 8 |				10 = home
			        |---|
			
			create 11 locations in a dictionary
			******************************************************************************
			*These could be read from a text file, so many different games can be created*
			******************************************************************************
			lstLocation.Add(new Location("short one word name suitable for the room/area",
			                             "start with 'a' then a suitable name to expand on the one used above",
			                             "description of the room/area, as long as you like",
			                             Locations to the N,E,S,W No exit = "", other exits use the short one word name e.g. "coridoor1"
			                             new List<Item>{dictItems["item name1"]})); <- List of any items found in this location 
			*/
			dictLocation.Add("bedroom",   new Location("bedroom",
			                             "a hotel room",
			                             "smelling of stale cigarettes (the room, not you..)",
			                             "", "coridoor", "", "",
			                             new List<Item> { dictItems["torch"], dictItems["hotel card"] }, dictItems["hotel card"]));
			dictLocation.Add("coridoor", new Location("coridoor",
			                             "a coridoor",
			                             "a dimly lit passage with a stained carpet",
			                             "", "lift", "hallway", "bedroom"));
			dictLocation.Add("lift",		 new Location("lift",
			                             "a lift",
			                             "a dangerous structure with a hand-operated sliding door",
			                             "", "", "", "coridoor",
			                             new List<Item>{ dictItems["LED torch"] }));  
			dictLocation.Add("hallway",  new Location("hallway",
			                             "a hallway",
			                             "a narrow area with peeling wallpaper",
			                             "coridoor", "", "diner", ""));
			dictLocation.Add("diner",    new Location("diner",
			                             "the dining room",
			                             "a large room filled with plastic tables and chairs",
			                             "hallway", "", "bar", "store"));
			dictLocation.Add("store",    new Location("store",
			                             "a store-room",
			                             "a broom cupboard filled with junk",
			                             "", "diner", "", "",
			                             new List<Item> { dictItems["knife"], dictItems["iron key"] }));
			dictLocation.Add("bar",      new Location("bar",
			                             "'The Snug Bar'",
			                             "very classy, even has sawdust on the floor",
			                             "diner", "games", "", ""));
			dictLocation.Add("games",     new Location("games",
			                             "the games lounge",
			                             "a flickering light illuminates the moth-eaten sofas",
			                             "", "reception", "wc", "bar"));
			dictLocation.Add("wc",        new Location("wc",
			                             "the toilets",
			                             "a shared facility in worse condition than found at Glastonbury",
			                             "games", "", "", ""));
			dictLocation.Add("reception", new Location("reception",
			                             "the reception area",
			                             "a delightful receptionist states: Computer says 'No'",
			                             "", "home", "", "games"));
			dictLocation.Add("home",      new Location("home",
			                             "your bed at home",
			                             "in a cold sweat, after a bad dream...",
			                             "", "", "", ""));
			
			//Create the player
			
			Player.Name 	= "Inksaver";					// put your name here or write a starting form to obtain Player details
			Player.Health 	= 100;							// not used so far. Enemies are needed!
			Player.Strength = 50;							// not used so far. Enemies are needed!
			Player.AddToInventory(dictItems["aspirin"]);    // Add aspirin to the player's inventory
			Player.GetItemFromInventory(dictItems["note"]); // Put note in player's hand
			currentLocation = dictLocation["bedroom"];		// Set starting location (bedroom)
			Intro();										// show intro text
			Play();											// Play the game
		}
		private void DisableButton(Button button)
		{
			button.Enabled = false;
			button.BackColor = Color.LightGray;
			button.ForeColor = Color.Gainsboro;
		}
		private void DisplayCompass()
		{          
			DisableButton(btnNorth);
			DisableButton(btnEast);
			DisableButton(btnSouth);
			DisableButton(btnWest);
			if (NorthExit != "") { EnableButton(btnNorth);}
			if (EastExit  != "") { EnableButton(btnEast); }
			if (SouthExit != "") { EnableButton(btnSouth);}
			if (WestExit  != "") { EnableButton(btnWest); }
		}
		private void DisplayHand()
		{
			if(Player.ItemInHand != null)
			{
				lblHand.Text = Player.Name + " is holding: " + Player.ItemInHand.Name;
				btnDrop.Enabled = true;
				btnReturn.Enabled = true;
				btnUse.Enabled = true;
			}
			else
			{
				lblHand.Text = Player.Name + " is holding: nothing";
				btnDrop.Enabled = false;
				btnReturn.Enabled = false;
				btnUse.Enabled = false;
			}
		}
		private void DisplayItemList()
		{
			lblLocationItems.Text = "Items in " + currentLocation.DisplayName;
			lstLocation.Items.Clear();
			btnExamine.Enabled = false;
			btnTake.Enabled = false;
			if(currentLocation.Items.Count > 0)
			{
				foreach (KeyValuePair<string, Item> item in currentLocation.Items)
				{
					lstLocation.Items.Add(item.Value.Name);
				}
				btnExamine.Enabled = true;
				btnTake.Enabled = true;
			}
		}
		private void DisplayInventory()
		{
			lstInventory.Items.Clear();
			if(Player.Inventory.Count > 0)
			{
				foreach (KeyValuePair<string,Item> item in Player.Inventory)
				{
					lstInventory.Items.Add(item.Value.Name);
				}
			}
		}
		private void DisplayStory(string AddText, bool clear = false)
		{
			if (clear)
			{
				lblGame.Text = string.Empty;
			}
			lblGame.Text += AddText;
		}
		private void DropItem()
		{
			DisplayStory($"\nYou drop the {Player.ItemInHand.Name}");
			currentLocation.AddItem(Player.ItemInHand);
			Player.ItemInHand = null;
			DisplayItemList();
			DisplayHand();
		}
		private void EnableButton(Button button)
		{
			button.Enabled = true;
			button.BackColor = Color.Green;;
			button.ForeColor = Color.Black;
		}
		private void ExamineItem(ListBox lb)
		{
			if (lb.SelectedItems.Count > 0)
			{
				Item item = dictItems[lb.SelectedItem.ToString()]; // find the item by name
				DisplayStory($"\nYou look at the {item.Name} ");
				DisplayStory(item.Description);           
			}
		}
		private void Go(string Direction)
		{
			//has to be an exit here else button would not be enabled
			Location checkLocation = currentLocation;
			switch (Direction)
			{
			// CheckExit will change currentLocation to new value
			case "north":	CheckExit(dictLocation[NorthExit]);	break;
			case "east":	CheckExit(dictLocation[EastExit]);	break;
			case "south":	CheckExit(dictLocation[SouthExit]);	break;
			case "west":	CheckExit(dictLocation[WestExit]);	break;
			}
			if (checkLocation != currentLocation)
			{
				Play();
			}
		}
		private void HoldItem()
		{
			if (lstInventory.SelectedItems.Count > 0)
			{
				//return the current item held to inventory and take new item
				Player.GetItemFromInventory(lstInventory.SelectedItem.ToString());
				DisplayStory($"\nYou get the {Player.ItemInHand.Name} from your backpack");
				DisplayInventory();
				DisplayHand();
			}
		}
		private void Intro()
		{
			lblGame.Text = 	"Welcome to the worst adventure game you have ever played.\n" + 
							"You have a serious hangover and you have no idea where you are.\n\n" + 
							"Check your inventory. Anything in your sweaty hand?\n\n" + 
							"Take a look around and see what you can do...\n";
		}
		private void Play()
		{
			//fill description
			DisplayStory($"You are in {currentLocation.DisplayName}, {currentLocation.Description}" , true);
			this.Text = $"{currentLocation.DisplayName} - {currentLocation.Description}";
			NorthExit = currentLocation.LocationToNorth;
			EastExit  = currentLocation.LocationToEast;
			SouthExit = currentLocation.LocationToSouth;
			WestExit  = currentLocation.LocationToWest;
			DisplayItemList();
			DisplayInventory();
			DisplayCompass();
			DisplayHand();       
		}
		private void ReturnItem()
		{
			Player.ReturnItemToInventory();
			DisplayInventory();
			DisplayHand(); 
		}
		private void TakeItem()
		{
			if (lstLocation.SelectedItems.Count > 0)
			{
				// Take item from location; Put in inventory
				// add to inventory items
				Player.AddToInventory(dictItems[lstLocation.SelectedItem.ToString()]);
				DisplayStory($"\nYou take the {lstLocation.SelectedItem}");
				// remove item from current location
				currentLocation.RemoveItem(lstLocation.SelectedItem.ToString());
				DisplayItemList();
				DisplayInventory();
			}
		}
		private void UseItem()
		{
			/// Use the item currently in your hand
			/// If it is a key, allow use into a location
			/// If it is a weapon, use it against an enemy
			/// If it is a note, read it
			Item currentItem = Player.ItemInHand;
			if (currentItem == null)
			{
				DisplayStory("\nYou are not holding anything in your hand!", true);
			}
			else
			{
				if (currentItem is Weapon)
				{
					Weapon weapon = (Weapon)currentItem;
					DisplayStory(	$"\nYou use the {weapon.Name} to scratch the wall with" +
								$" {weapon.Damage} points of damage");

				}
				else if (currentItem is Potion)
				{
					Potion potion = (Potion)currentItem;
					DisplayStory( $"\nYou use the {potion.Name} for a health boost of" +
								$" {potion.Healing} and feel so much better!");
					Player.ItemInHand = null;
					DisplayHand();
				}
				else if(currentItem is Note)
				{
					Note note = (Note)currentItem;
					DisplayStory(	$"\nThe {note.Name} in your hand is " +
								$"{note.Description} which reads:\n'{note.Message}'");
				}
			}

		}
		#endregion
	}
}
