﻿using System.Collections.Generic;

namespace Text_Adventure_Tutorial
{
	public static class Player
	{
		/// <summary>
		/// There is only one player, so a static class is used
		/// There is no real need to encapsulate variables in a static class
		/// but some have been given auto-properties to show how it can be done
		/// </summary>

		#region Public Properties
		public static int Strength;                 // using a public read/write variable
		public static string Name { get; set; }     // using  an auto-property
		public static int Health { get; set; }       // using  an auto-property
		public static Dictionary<string, Item> Inventory = new Dictionary<string, Item>();
		public static Item ItemInHand;              // using a public read/write variable
        #endregion
        #region Static Constructor
        static Player()
        {
			/// Static classes do not usually require a constructor
			/// They run once only, when the first reference to the 
			/// class is made, and can be used to initialise data, etc
        }
        #endregion
        #region Public Methods
        public static void AddToInventory(Item item)
		{
			Inventory.Add(item.Name, item);
		}
		public static void GetItemFromInventory(Item item)
		{
			/// Overloaded function, This version takes an Item parameter.
			/// It can also accept a string parameter in another overload
			/// which sends it back here as an Item for processing
			ReturnItemToInventory();
			ItemInHand = item;
			Inventory.Remove(item.Name);
		}
		public static void GetItemFromInventory(string itemName)
		{
			/// Overload for this function, takes string parameter
			/// Converts it to Item object
			/// Then passes it to the relevant overload
			GetItemFromInventory(Inventory[itemName]);
		}
		public static void ReturnItemToInventory()
		{
			if(ItemInHand != null)
			{
				Inventory.Add(ItemInHand.Name, ItemInHand);
			}
			ItemInHand = null;
		}
        #endregion
    }
}
